/*
	$Id: DalDynamic.h 1193 2018-11-20 11:58:19Z john $

	This file is part of the RFC DAL.

	The RFC DAL is Copyright (c) 2004 - 2007, The 4c Company.

	The RFC DAL is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC DAL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DalDynamic_h__
#define __DalDynamic_h__

#include <librfcdal/DalDatabase.h>

#if defined (RFCDALDYN_EXPORT)
#	undef RFCDALDYN_EXPORT
#endif /* RFCDALDYN_EXPORT */

#if defined (LINUX)
#	define RFCDALDYN_EXPORT
#elif defined (_WINDOWS)
#	include <windows.h>
#	if defined (LIBRFCDALDYN_EXPORTS)
#		define RFCDALDYN_EXPORT __declspec( dllexport )
#	else
#		define RFCDALDYN_EXPORT __declspec( dllimport )
#	endif /* LIBRFCDALDBODBC_EXPORTS */
#endif /* LINUX */

namespace RfcDal
{
	class RFCDALDYN_EXPORT DalDynamic
	{
		public:
			enum Adaptor{ unknown = -1, mysql, odbc, sqlite, db2, informix, oracle, postgres }; 

			class RFCDALDYN_EXPORT Engine
			{
				private:
					std::string _strObject;
					std::string _strAdaptorName;
					std::string _strDescription;
					Adaptor _adaptor;

				public:
					Engine();
					Engine( const std::string& strObject, const std::string& strAdaptorName, const std::string& strDescription, Adaptor adaptor );
					Engine( const Engine& engine );
					virtual ~Engine();

					const Engine& operator=( const Engine& engine );

					const std::string& object() const;
					const std::string& adaptorName() const;
					const std::string& description() const;
					Adaptor adaptor() const;
			};

			class RFCDALDYN_EXPORT SharedObject
			{
				private:
					Adaptor _adaptor;
#if defined (LINUX)
					void* _handle;
#elif defined (_WINDOWS)
					HMODULE _handle;
#endif /* LINUX */
					DalDatabase*(*_dalDatabase)( void );
					int _nUsageCount;

				public:
					SharedObject();
					SharedObject( Adaptor adaptor );
					SharedObject( const SharedObject& object );
					virtual ~SharedObject();

					const SharedObject& operator=( const SharedObject& object );

					Adaptor adaptor() const;
#if defined (LINUX)
					void* handle();
#elif defined (_WINDOWS)
					HMODULE handle();
#endif /* LINUX */
					DalDatabase& dalDatabase();
					void terminate( DalDatabase& db );

			};

			class EngineMap : public std::map<Adaptor, Engine>
			{
			};
			class DatabaseMap : public std::map<DalDatabase*, SharedObject*> 
			{
			};
			class ObjectMap : public std::map<Adaptor, SharedObject>
			{
			};

		private:
			static EngineMap _engineS;
			static DatabaseMap _databaseS;
			static ObjectMap _objectS;

		public:
			static const EngineMap& engines();
			static DalDatabase& dalDatabase( Adaptor adaptor );
			static DalDatabase& dalDatabase( const std::string& strAdaptorName );

			static DalDatabase& connectX( const std::string& strConnect, DalDatabase::ConnectionParameterMap& map = DalDatabase::_mapDefault );
			static DalDatabase& connectX( DalDatabase::ConnectionParameterMap& map );

			static void terminate( DalDatabase& db );

	};
};

#endif /* !__rfcdaldyn_h__ */
